/**
 * Java
 * 
 * Copyright 2013 IS2T. All rights reserved
 * For demonstration purpose only
 * IS2T PROPRIETARY. Use is subject to license terms.
 */
package com.is2t.example;

import java.util.Vector;

import ej.bon.EnqueuedWeakReference;
import ej.bon.ReferenceQueue;

public class ResourceManager {
    public static class OutOfResourcesException extends Exception {
    }

    public static interface ResourceHelper {
        void allocateResource() throws OutOfResourcesException;
        void cleanup();
    }

    private final ReferenceQueue queue = new ReferenceQueue();
    private final Vector activeReferenceList = new Vector();

    public void allocateResource(Object resourceUser, ResourceHelper helper) throws OutOfResourcesException {
        cleanup();
        try {
            helper.allocateResource();
        } catch (OutOfResourcesException e) {
            System.gc();
            cleanup();
            helper.allocateResource();
        }
        activeReferenceList.addElement(new ResourceReference(helper, resourceUser, queue));
    }

    private void cleanup() {
        ResourceReference removedReference;
        while((removedReference=(ResourceReference) queue.poll()) != null){
            activeReferenceList.removeElement(removedReference);
            removedReference.cleanup();
        }
    }

    private static class ResourceReference extends EnqueuedWeakReference {
        private final ResourceHelper helper;
    
        private ResourceReference(ResourceHelper helper, Object resourceUser, ReferenceQueue queue) {
            super(resourceUser, queue);
            this.helper = helper;
        }
    
        private void cleanup() {
            helper.cleanup();
        }
    }
}